## $Id: tests.py 18036 2025-03-11 19:29:27Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import os
from datetime import datetime, date, timedelta
from zope.component import createObject, getUtility
from zope.catalog.interfaces import ICatalog
from hurry.workflow.interfaces import IWorkflowState
from waeup.kofa.students.tests.test_browser import StudentsFullSetup
from waeup.kofa.applicants.tests.test_browser import ApplicantsFullSetup
from waeup.kofa.configuration import SessionConfiguration
from kofacustom.iuokada.testing import FunctionalLayer

# Also run tests that send requests to external servers?
#   If you enable this, please make sure the external services
#   do exist really and are not bothered by being spammed by a test programme.
EXTERNAL_TESTS = False

def external_test(func):
    if not EXTERNAL_TESTS:
        myself = __file__
        if myself.endswith('.pyc'):
            myself = myself[:-2]
        print "WARNING: external tests are skipped!"
        print "WARNING: edit %s to enable them." % myself
        return
    return func


class InterswitchTestsStudents(StudentsFullSetup):
    """Tests for the Interswitch payment gateway.
    """

    layer = FunctionalLayer

    def setUp(self):
        super(InterswitchTestsStudents, self).setUp()
        self.app['configuration']['2004'].interswitch_enabled = True

    def test_interswitch_form(self):
        self.app['configuration']['2004'].registration_fresh_fee = 7000.0
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.payments_path)
        IWorkflowState(self.student).setState('cleared')
        self.student.nationality = u'NG'
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(name="form.p_category").value = ['registration_fresh']
        self.browser.getControl(name="form.p_option").value = ['first']
        self.browser.getControl("Create payment").click()
        self.assertTrue('Payment created' in self.browser.contents)
        self.browser.open(self.payments_path)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]
        self.browser.getLink(self.value).click()
        self.assertTrue(
            '<span>7000.0</span>' in self.browser.contents)
        self.payment_url = self.browser.url
        self.browser.getLink("Pay via Interswitch", index=0).click()
        self.assertTrue('<input type="hidden" name="pay_item_id" value="102" />' in
                           self.browser.contents)
        self.assertEqual(self.student.current_mode, 'ug_ft')
        # 250 gateway charge have been added
        payment = self.student['payments'][self.value]
        self.assertEqual(payment.amount_auth, 7250)
        self.assertEqual(payment.net_amt, 7000)
        self.assertTrue(
            '<input type="hidden" name="amount" value="725000" />' in
            self.browser.contents)
        # WAeAC charge have been substracted
        self.assertTrue(
            'item_name="Registration Fee (Fresh)" item_amt="200000" bank_id="8" acct_num="2021420049"' in
            self.browser.contents)
        self.assertTrue(
            'item_name="WAeAC" item_amt="500000" bank_id="31" acct_num="0773411069"' in
            self.browser.contents)

    def test_interswitch_required_combi(self):
        # Student is a fresh non-science student
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.payments_path)
        IWorkflowState(self.student).setState('cleared')
        self.student.nationality = u'NG'
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(name="form.p_category").value = ['schoolfee']
        self.browser.getControl(name="form.p_option").value = ['first']
        self.browser.getControl("Create payment").click()
        self.assertTrue('Make either single payments or make a' in self.browser.contents)
        self.browser.getControl(name="form.p_category").value = ['required_combi']
        self.browser.getControl("Create payment").click()
        self.assertTrue('Student Health Insurance undefined.' in self.browser.contents)
        self.app['configuration']['2004'].registration_fresh_fee = 10000.0
        self.app['configuration']['2004'].book_fee = 10000.0
        self.app['configuration']['2004'].develop_fee = 10000.0
        self.app['configuration']['2004'].parentsconsult_fee = 10000.0
        self.app['configuration']['2004'].municipal_fresh_fee = 10000.0
        self.app['configuration']['2004'].matric_fee = 10000.0
        self.app['configuration']['2004'].waecneco_fee = 10000.0
        self.app['configuration']['2004'].jambver_fee = 10000.0
        self.app['configuration']['2004'].health_insurance_fee = 10000.0
        self.app['configuration']['2004'].id_card_fee = 10000.0
        self.app['configuration']['2004'].medical_screening_fee = 10000.0
        self.browser.getControl(name="form.p_category").value = ['required_combi']
        self.browser.getControl("Create payment").click()
        self.assertTrue('Payment created' in self.browser.contents)
        self.browser.open(self.payments_path)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]
        self.browser.getLink(self.value).click()
        self.assertTrue(
            '<span>110000.0</span>' in self.browser.contents)
        self.payment_url = self.browser.url
        self.browser.getLink("Pay via Interswitch", index=0).click()
        self.assertTrue('<input type="hidden" name="pay_item_id" value="102" />' in
                           self.browser.contents)
        self.assertEqual(self.student.current_mode, 'ug_ft')
        # 250 gateway charge have been added
        payment = self.student['payments'][self.value]
        self.assertEqual(payment.amount_auth, 110250)
        self.assertEqual(payment.net_amt, 110000)
        self.assertTrue(
            '<input type="hidden" name="amount" value="11025000" />' in
            self.browser.contents)
        self.assertTrue(
            'item_id="6" item_name="registration_fresh" item_amt="500000" bank_id="31" acct_num="0040621193"' in
            self.browser.contents)
        self.assertTrue(
            'item_id="12" item_name="parentsconsult" item_amt="1000000" bank_id="117" acct_num="1228747029"' in
            self.browser.contents)

    def test_interswitch_form_ticket_expired(self):
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        acc_payment = createObject('waeup.StudentOnlinePayment')
        acc_payment.p_state = 'unpaid'
        acc_payment.p_category = 'registration'
        acc_payment.p_id = 'xyz'
        acc_payment.pay_item_id = '123'
        acc_payment.amount_auth = 1000.0
        self.student['payments']['xyz'] = acc_payment
        self.browser.open(self.payments_path + '/xyz')
        self.browser.getLink("Pay via Interswitch", index=0).click()
        self.assertTrue('<input type="hidden" name="pay_item_id" value="102" />' in
                           self.browser.contents)
        self.assertMatches('...Total Amount Authorized:...',
                           self.browser.contents)
        self.assertEqual(self.student.current_mode, 'ug_ft')
        # 250 gateway charge have been added
        self.assertTrue(
            '<input type="hidden" name="amount" value="125000" />' in
            self.browser.contents)
        self.assertEqual(acc_payment.amount_auth, 1250)
        self.assertEqual(acc_payment.net_amt, 1000)
        delta = timedelta(days=8)
        acc_payment.creation_date -= delta
        self.browser.open(self.payments_path + '/xyz')
        self.browser.getLink("Pay via Interswitch", index=0).click()
        self.assertMatches(
            '...This payment ticket is too old. Please create a new ticket...',
            self.browser.contents)
        delta = timedelta(days=2)
        acc_payment.creation_date += delta
        self.browser.open(self.payments_path + '/xyz')
        self.browser.getLink("Pay via Interswitch", index=0).click()
        self.assertMatches('...Total Amount Authorized:...',
                           self.browser.contents)

    @external_test
    def test_webservice(self):
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.payments_path)
        IWorkflowState(self.student).setState('cleared')
        self.student.nationality = u'NG'
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(name="form.p_category").value = ['schoolfee']
        self.browser.getControl("Create payment").click()
        self.assertMatches('...ticket created...',
                           self.browser.contents)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]
        self.browser.getLink(self.value).click()
        self.payment_url = self.browser.url
        # First we have open InterswitchPageStudent to set provider_amt
        # and gateway_amt
        self.browser.open(self.payment_url + '/goto_interswitch')
        # Now we can call the webservice
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertMatches('...Unsuccessful callback...',
                          self.browser.contents)
        # The payment is now in state failed ...
        self.assertMatches('...<span>Failed</span>...',
                          self.browser.contents)
        # ... and the catalog has been updated
        cat = getUtility(ICatalog, name='payments_catalog')
        results = list(
            cat.searchResults(p_state=('failed', 'failed')))
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0].p_state, 'failed')

        # Let's replace the p_id with a valid p_id of the Uniben
        # live system. This is definitely not an appropriate
        # solution for testing, but we have no choice since
        # Interswitch doesn't provide any interface
        # for testing.
        payment = self.student['payments'][self.value]
        payment.p_id = 'p3547789850240'
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertMatches('...Callback amount does not match...',
                          self.browser.contents)
        # The payment is now in state failed ...
        self.assertMatches('...<span>Failed</span>...',
                          self.browser.contents)
        # Let's replace the amount autorized with the amount of the
        # live system payment
        payment.amount_auth = payment.r_amount_approved
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertMatches('...Successful payment...',
                          self.browser.contents)
        # The payment is now in state paid ...
        self.assertMatches('...<span>Paid</span>...',
                          self.browser.contents)
        # ... and the catalog has been updated
        cat = getUtility(ICatalog, name='payments_catalog')
        results = list(
            cat.searchResults(p_state=('paid', 'paid')))
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0].p_state, 'paid')
        # Approval is logged in students.log ...
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'students.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            'zope.mgr - '
            'kofacustom.iuokada.interswitch.browser.CustomInterswitchPaymentRequestWebservicePageStudent - '
            'X1000000 - successful schoolfee payment: p3547789850240\n'
            in logcontent)
        # ... and in payments.log
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'payments.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            '"zope.mgr",X1000000,p3547789850240,schoolfee,'
            '12000.0,00,0.0,150.0,0.0,,,\n'
            in logcontent)


class InterswitchTestsApplicants(ApplicantsFullSetup):
    """Tests for the Interswitch payment gateway.
    """

    layer = FunctionalLayer

    def fill_correct_values(self):
        # Fill the edit form with suitable values
        self.browser.getControl(name="form.firstname").value = 'John'
        self.browser.getControl(name="form.middlename").value = 'Anthony'
        self.browser.getControl(name="form.lastname").value = 'Tester'
        self.browser.getControl(name="custom.course1").value = ['CERT1']
        self.browser.getControl(name="form.date_of_birth").value = '09/09/1988'
        self.browser.getControl(name="form.sex").value = ['m']
        self.browser.getControl(name="form.email").value = 'xx@yy.zz'
        self.browser.getControl(name="form.perm_address").value = 'Adresse'

    def setUp(self):
        super(InterswitchTestsApplicants, self).setUp()
        configuration = SessionConfiguration()
        configuration.academic_session = datetime.now().year - 2
        self.app['configuration'].addSessionConfiguration(configuration)
        self.configuration = configuration
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.manage_path)
        #IWorkflowState(self.student).setState('started')
        self.fill_correct_values()
        self.applicantscontainer.application_fee = 1000.0
        self.browser.getControl(name="form.nationality").value = ['NG']
        self.browser.getControl(name="transition").value = ['start']
        self.browser.getControl("Save").click()
        self.browser.getControl("Add online").click()
        self.assertMatches('...Payment created...',
                           self.browser.contents)
        #ctrl = self.browser.getControl(name='val_id')
        #value = ctrl.options[0]
        #self.browser.getLink(value).click()
        self.assertMatches('...Amount Authorized...',
                           self.browser.contents)
        self.assertMatches(
            '...<span>1000.0</span>...',
            self.browser.contents)
        self.payment_url = self.browser.url


    def test_interswitch_form(self):
        # Manager can access InterswitchForm
        self.browser.getLink("Pay via Interswitch", index=0).click()
        self.assertMatches('...Total Amount Authorized:...',
                           self.browser.contents)
        self.assertMatches(
            '...<input type="hidden" name="amount" value="100000" />...',
            self.browser.contents)
        delta = timedelta(days=8)
        self.applicant.values()[0].creation_date -= delta
        self.browser.open(self.payment_url)
        self.browser.getLink("Pay via Interswitch", index=0).click()
        self.assertMatches(
            '...This payment ticket is too old. Please create a new ticket...',
            self.browser.contents)

    @external_test
    def test_webservice(self):

        self.browser.open(self.payment_url + '/request_webservice')
        self.assertMatches('...Unsuccessful callback...',
                          self.browser.contents)
        # The payment is now in state failed
        self.assertMatches('...<span>Failed</span>...',
                          self.browser.contents)
